@@ -29,6 +29,7 @@
 
 #include "angband.h"
 #include "cave.h"
+#include "datafile.h"
 #include "game-event.h"
 #include "game-input.h"
 #include "game-world.h"
@@ -36,14 +37,14 @@
 #include "init.h"
 #include "math.h"
 #include "mon-make.h"
+#include "mon-move.h"
 #include "mon-spell.h"
 #include "monster.h"
-#include "obj-identify.h"
 #include "obj-tval.h"
 #include "obj-util.h"
 #include "object.h"
-#include "parser.h"
 #include "player-history.h"
+#include "player-util.h"
 #include "trap.h"
 #include "z-queue.h"
 #include "z-type.h"
@@ -53,8 +54,9 @@
  */
 struct pit_profile *pit_info;
 struct vault *vaults;
-struct cave_profile *cave_profiles;
-
+static struct cave_profile *cave_profiles;
+struct dun_data *dun;
+struct room_template *room_templates;
 
 static const struct {
 	const char *name;
@@ -165,6 +167,7 @@ static enum parser_error parse_profile_room(struct parser *p) {
 	if (i == N_ELEMENTS(room_builders))
 		return PARSE_ERROR_NO_ROOM_FOUND;
 	r->builder = room_builders[i].builder;
+	r->rating = parser_getint(p, "rating");
     r->height = parser_getint(p, "height");
     r->width = parser_getint(p, "width");
     r->level = parser_getint(p, "level");
@@ -190,13 +193,13 @@ static struct parser *init_parse_profile(void) {
 	parser_reg(p, "params int block int rooms int unusual int rarity", parse_profile_params);
 	parser_reg(p, "tunnel int rnd int chg int con int pen int jct", parse_profile_tunnel);
 	parser_reg(p, "streamer int den int rng int mag int mc int qua int qc", parse_profile_streamer);
-	parser_reg(p, "room sym name int height int width int level int pit int rarity int cutoff", parse_profile_room);
+	parser_reg(p, "room sym name int rating int height int width int level int pit int rarity int cutoff", parse_profile_room);
 	parser_reg(p, "cutoff int cutoff", parse_profile_cutoff);
 	return p;
 }
 
 static errr run_parse_profile(struct parser *p) {
-	return parse_file(p, "dungeon_profile");
+	return parse_file_quit_not_found(p, "dungeon_profile");
 }
 
 static errr finish_parse_profile(struct parser *p) {
@@ -415,7 +418,7 @@ static struct parser *init_parse_room(void) {
 }
 
 static errr run_parse_room(struct parser *p) {
-	return parse_file(p, "room_template");
+	return parse_file_quit_not_found(p, "room_template");
 }
 
 static errr finish_parse_room(struct parser *p) {
@@ -562,7 +565,7 @@ struct parser *init_parse_vault(void) {
 }
 
 static errr run_parse_vault(struct parser *p) {
-	return parse_file(p, "vault");
+	return parse_file_quit_not_found(p, "vault");
 }
 
 static errr finish_parse_vault(struct parser *p) {
@@ -662,15 +665,15 @@ static void place_feeling(struct chunk *c)
  * Calculate the level feeling for objects.
  * \param c is the cave where the feeling is being measured
  */
-static int calc_obj_feeling(struct chunk *c)
+static int calc_obj_feeling(struct chunk *c, struct player *p)
 {
 	u32b x;
 
 	/* Town gets no feeling */
 	if (c->depth == 0) return 0;
 
-	/* Artifacts trigger a special feeling when preserve=no */
-	if (c->good_item && OPT(birth_no_preserve)) return 10;
+	/* Artifacts trigger a special feeling when they can be easily lost */
+	if (c->good_item && OPT(p, birth_lose_arts)) return 10;
 
 	/* Check the loot adjusted for depth */
 	x = c->obj_rating / c->depth;
@@ -701,7 +704,7 @@ static int calc_mon_feeling(struct chunk *c)
 	if (c->depth == 0) return 0;
 
 	/* Check the monster power adjusted for depth */
-	x = c->mon_rating / (c->depth * c->depth);
+	x = c->mon_rating / c->depth;
 
 	if (x > 7000) return 1;
 	if (x > 4500) return 2;
@@ -724,10 +727,10 @@ bool labyrinth_check(int depth)
 	int chance = 2;
 
 	/* If we're too shallow then don't do it */
-	if (depth < 13) return FALSE;
+	if (depth < 13) return false;
 
 	/* Don't try this on quest levels, kids... */
-	if (is_quest(depth)) return FALSE;
+	if (is_quest(depth)) return false;
 
 	/* Certain numbers increase the chance of having a labyrinth */
 	if (depth % 3 == 0) chance += 1;
@@ -737,10 +740,10 @@ bool labyrinth_check(int depth)
 	if (depth % 13 == 0) chance += 1;
 
 	/* Only generate the level if we pass a check */
-	if (randint0(100) >= chance) return FALSE;
+	if (randint0(100) >= chance) return false;
 
 	/* Successfully ran the gauntlet! */
-	return TRUE;
+	return true;
 }
 
 /**
@@ -765,15 +768,15 @@ const struct cave_profile *find_cave_profile(char *name)
 
 /**
  * Choose a cave profile
- * \param depth is the depth of the cave the profile will be used to generate
+ * \param p is the player
  */
-const struct cave_profile *choose_profile(int depth)
+const struct cave_profile *choose_profile(struct player *p)
 {
 	const struct cave_profile *profile = NULL;
 
 	/* A bit of a hack, but worth it for now NRM */
 	if (player->noscore & NOSCORE_JUMPING) {
-		char name[30];
+		char name[30] = "";
 
 		/* Cancel the query */
 		player->noscore &= ~(NOSCORE_JUMPING);
@@ -787,19 +790,21 @@ const struct cave_profile *choose_profile(int depth)
 	}
 
 	/* Make the profile choice */
-	if (depth == 0)
+	if (p->depth == 0) {
 		profile = find_cave_profile("town");
-	else if (is_quest(depth))
+	} else if (is_quest(p->depth) && !OPT(p, birth_levels_persist)) {
 		/* Quest levels must be normal levels */
 		profile = find_cave_profile("classic");
-	else if (labyrinth_check(depth))
+	} else if (labyrinth_check(p->depth)) {
 		profile = find_cave_profile("labyrinth");
-	else {
-		int perc = randint0(100);
+	} else if ((p->depth >= 10) && (p->depth < 40) && one_in_(40)) {
+		profile = find_cave_profile("moria");
+	} else {
+		int pick = randint0(200);
 		size_t i;
 		for (i = 0; i < z_info->profile_max; i++) {
 			profile = &cave_profiles[i];
-			if (profile->cutoff >= perc) break;
+			if (profile->cutoff >= pick) break;
 		}
 	}
 
@@ -812,31 +817,105 @@ const struct cave_profile *choose_profile(int depth)
 	return NULL;
 }
 
+/**
+ * Get information for constructing stairs in the correct places
+ */
+static void get_join_info(struct player *p, struct dun_data *dun)
+{
+	struct level *lev = NULL;
+
+	/* Check level above */
+	lev = level_by_depth(p->depth - 1);
+	if (lev) {
+		struct chunk *check = chunk_find_name(lev->name);
+		if (check) {
+			struct connector *join = check->join;
+			while (join) {
+				if (join->feat == FEAT_MORE) {
+					struct connector *new = mem_zalloc(sizeof *new);
+					new->grid.y = join->grid.y;
+					new->grid.x = join->grid.x;
+					new->feat = FEAT_LESS;
+					new->next = dun->join;
+					dun->join = new;
+				}
+				join = join->next;
+			}
+		}
+	}
+
+	/* Check level below */
+	lev = level_by_depth(p->depth + 1);
+	if (lev) {
+		struct chunk *check = chunk_find_name(lev->name);
+		if (check) {
+			struct connector *join = check->join;
+			while (join) {
+				if (join->feat == FEAT_LESS) {
+					struct connector *new = mem_zalloc(sizeof *new);
+					new->grid.y = join->grid.y;
+					new->grid.x = join->grid.x;
+					new->feat = FEAT_MORE;
+					new->next = dun->join;
+					dun->join = new;
+				}
+				join = join->next;
+			}
+		}
+	}
+}
+
+/**
+ * Check the size of the level above or below the next level to be generated
+ * to make sure stairs can connect
+ */
+static void	get_min_level_size(struct chunk *check, int *min_height,
+							   int *min_width, bool above)
+{
+	struct connector *join = check->join;
+
+	while (join) {
+		if ((above && (join->feat == FEAT_MORE)) ||
+			(!above && (join->feat == FEAT_LESS))) {
+			*min_height = MAX(*min_height, join->grid.y + 2);
+			*min_width = MAX(*min_width, join->grid.x + 2);
+		}
+		join = join->next;
+	}
+}
+
+
+/**
+ * Store a dungeon level for reloading
+ */
+static void cave_store(struct chunk *c, bool known, bool keep_all)
+{
+	struct chunk *stored;
+	if (keep_all) {
+		stored = c;
+	} else {
+		stored = chunk_write(c);
+	}
+	if (stored->name) {
+		string_free(stored->name);
+	}
+	stored->name = string_make(level_by_depth(c->depth)->name);
+	if (known) {
+		stored->name = string_append(stored->name, " known");
+	}
+	stored->turn = turn;
+	chunk_list_add(stored);
+}
+
+
 /**
  * Clear the dungeon, ready for generation to begin.
  */
 static void cave_clear(struct chunk *c, struct player *p)
 {
-	int x, y;
-
 	/* Clear the monsters */
 	wipe_mon_list(c, p);
 
-	/* Deal with artifacts */
-	for (y = 0; y < c->height; y++) {
-		for (x = 0; x < c->width; x++) {
-			struct object *obj = square_object(c, y, x);
-			while (obj) {
-				if (obj->artifact) {
-					if (!OPT(birth_no_preserve) && !object_was_sensed(obj))
-						obj->artifact->created = FALSE;
-					else
-						history_lose_artifact(obj->artifact);
-				}
-				obj = obj->next;
-			}
-		}
-	}
 	/* Free the chunk */
 	cave_free(c);
 }
@@ -845,26 +924,25 @@ static void cave_clear(struct chunk *c, struct player *p)
 /**
  * Generate a random level.
  *
- * Confusingly, this function also generate the town level (level 0).
- * \param c is the level we're going to end up with, in practice the global cave
+ * Confusingly, this function also generates the town level (level 0).
  * \param p is the current player struct, in practice the global player
+ * \return a pointer to the new level
  */
-void cave_generate(struct chunk **c, struct player *p)
+static struct chunk *cave_generate(struct player *p, int height, int width)
 {
 	const char *error = "no generation";
-	int y, x, tries = 0;
-	struct chunk *chunk;
-
-	assert(c);
+	int i, tries = 0;
+	struct chunk *chunk = NULL;
 
 	/* Generate */
 	for (tries = 0; tries < 100 && error; tries++) {
+		int y, x;
 		struct dun_data dun_body;
 
 		error = NULL;
 
 		/* Mark the dungeon as being unready (to avoid artifact loss, etc) */
-		character_dungeon = FALSE;
+		character_dungeon = false;
 
 		/* Allocate global data (will be freed when we leave the loop) */
 		dun = &dun_body;
@@ -872,12 +950,19 @@ void cave_generate(struct chunk **c, struct player *p)
 		dun->door = mem_zalloc(z_info->level_door_max * sizeof(struct loc));
 		dun->wall = mem_zalloc(z_info->wall_pierce_max * sizeof(struct loc));
 		dun->tunn = mem_zalloc(z_info->tunn_grid_max * sizeof(struct loc));
+		dun->join = NULL;
+
+		/* Get connector info for persistent levels */
+		if (OPT(p, birth_levels_persist)) {
+			get_join_info(p, dun);
+		}
 
 		/* Choose a profile and build the level */
-		dun->profile = choose_profile(p->depth);
-		chunk = dun->profile->builder(p);
+		dun->profile = choose_profile(p);
+		chunk = dun->profile->builder(p, height, width);
 		if (!chunk) {
 			error = "Failed to find builder";
+			mem_free(dun->join);
 			mem_free(dun->cent);
 			mem_free(dun->door);
 			mem_free(dun->wall);
@@ -887,29 +972,43 @@ void cave_generate(struct chunk **c, struct player *p)
 
 		/* Ensure quest monsters */
 		if (is_quest(chunk->depth)) {
-			int i;
-			for (i = 1; i < z_info->r_max; i++) {
-				struct monster_race *race = &r_info[i];
-				int y, x;
-				
+			int i2;
+			for (i2 = 1; i2 < z_info->r_max; i2++) {
+				struct monster_race *race = &r_info[i2];
+				int y2, x2;
+
 				/* The monster must be an unseen quest monster of this depth. */
 				if (race->cur_num > 0) continue;
 				if (!rf_has(race->flags, RF_QUESTOR)) continue;
 				if (race->level != chunk->depth) continue;
 	
 				/* Pick a location and place the monster */
-				find_empty(chunk, &y, &x);
-				place_new_monster(chunk, y, x, race, TRUE, TRUE, ORIGIN_DROP);
+				find_empty(chunk, &y2, &x2);
+				place_new_monster(chunk, y2, x2, race, true, true, ORIGIN_DROP);
 			}
 		}
 
-		/* Clear generation flags. */
+		/* Clear generation flags, add connecting info */
 		for (y = 0; y < chunk->height; y++) {
 			for (x = 0; x < chunk->width; x++) {
 				sqinfo_off(chunk->squares[y][x].info, SQUARE_WALL_INNER);
 				sqinfo_off(chunk->squares[y][x].info, SQUARE_WALL_OUTER);
 				sqinfo_off(chunk->squares[y][x].info, SQUARE_WALL_SOLID);
 				sqinfo_off(chunk->squares[y][x].info, SQUARE_MON_RESTRICT);
+
+				if (square_isstairs(chunk, y, x)) {
+					size_t n;
+					struct connector *new = mem_zalloc(sizeof *new);
+					new->grid.y = y;
+					new->grid.x = x;
+					new->feat = square_feat(chunk, y, x)->fidx;
+					new->info = mem_zalloc(SQUARE_SIZE * sizeof(bitflag));
+					for (n = 0; n < SQUARE_SIZE; n++) {
+						new->info[n] = chunk->squares[y][x].info[n];
+					}
+					new->next = chunk->join;
+					chunk->join = new;
+				}
 			}
 		}
 
@@ -918,10 +1017,13 @@ void cave_generate(struct chunk **c, struct player *p)
 			error = "too many monsters";
 
 		if (error) {
-			ROOM_LOG("Generation restarted: %s.", error);
+			if (OPT(p, cheat_room)) {
+				msg("Generation restarted: %s.", error);
+			}
 			cave_clear(chunk, p);
 		}
 
+		mem_free(dun->join);
 		mem_free(dun->cent);
 		mem_free(dun->door);
 		mem_free(dun->wall);
@@ -930,46 +1032,253 @@ void cave_generate(struct chunk **c, struct player *p)
 
 	if (error) quit_fmt("cave_generate() failed 100 times!");
 
-	/* Free the old cave, use the new one */
-	if (*c)
-		cave_clear(*c, p);
-	*c = chunk;
-
 	/* Place dungeon squares to trigger feeling (not in town) */
-	if (player->depth)
-		place_feeling(*c);
-
-	/* Save the town */
-	else if (!chunk_find_name("Town")) {
-		struct chunk *town = chunk_write(0, 0, z_info->town_hgt,
-										 z_info->town_wid, FALSE, FALSE, FALSE);
-		town->name = string_make("Town");
-		chunk_list_add(town);
+	if (p->depth) {
+		place_feeling(chunk);
 	}
 
-	(*c)->feeling = calc_obj_feeling(*c) + calc_mon_feeling(*c);
+	chunk->feeling = calc_obj_feeling(chunk, p) + calc_mon_feeling(chunk);
 
 	/* Validate the dungeon (we could use more checks here) */
-	chunk_validate_objects(*c);
+	chunk_validate_objects(chunk);
+
+	/* Allocate new known level, light it if requested */
+	p->cave = cave_new(chunk->height, chunk->width);
+	p->cave->depth = chunk->depth;
+	p->cave->objects = mem_realloc(p->cave->objects, (chunk->obj_max + 1)
+								   * sizeof(struct object*));
+	p->cave->obj_max = chunk->obj_max;
+	for (i = 0; i <= p->cave->obj_max; i++) {
+		p->cave->objects[i] = NULL;
+	}
+	if (p->upkeep->light_level) {
+		wiz_light(chunk, p, false);
+		p->upkeep->light_level = false;
+	}
+
+	chunk->turn = turn;
+
+	return chunk;
+}
+
+static void sanitize_player_loc(struct chunk *c, struct player *p)
+{
+	/* TODO potential problem: stairs in vaults? */
+	
+	/* allow direct transfer if target location is teleportable */
+	if (square_in_bounds_fully(c, p->py, p->px)
+			&& square_isarrivable(c, p->py, p->px)
+			&& !square_isvault(c, p->py, p->px)) {
+		return;
+	}
+	
+	/* TODO should use something similar to teleport code, but this will
+	 *  do for now as a quick'n dirty fix
+	 */
+	int tx, ty; // test locations
+	int ix, iy; // initial location
+	int vx=1, vy=1; // fallback vault location
+	int try = 1000; // attempts
+
+	/* a bunch of random locations */
+	while (try) {
+		try = try - 1;
+		tx = randint0(c->width-1) + 1;
+		ty = randint0(c->height-1) + 1;
+		if (square_isempty(c, ty, tx)
+				&& !square_isvault(c, ty, tx)) {
+			p->py = ty;
+			p->px = tx;
+			return;
+		}
+	}
+	
+	/* whelp, that didnt work */
+	ix = randint0(c->width-1) + 1;
+	iy = randint0(c->height-1) + 1;
+	ty = iy;
+	tx = tx + 1;
+	if (tx >= c->width - 1) {
+		tx = 1;
+		ty = ty + 1;
+		if (ty >= c->height -1) {
+			ty = 1;
+		}
+	}
+	
+	while (1) {		//until full loop through dungeon
+		if (square_isempty(c, ty, tx)) {
+			if (!square_isvault(c, ty, tx)) {
+				// ok location
+				p->py = ty;
+				p->px = tx;
+				return;
+			}
+			// vault, but lets remember it just in case
+			vy = ty;
+			vx = tx;
+		}
+		// oops tried *every* tile...
+		if (tx == ix && ty == iy) {
+			break;
+		}
+		tx = tx + 1;
+		if (tx >= c->width - 1) {
+			tx = 1;
+			ty = ty + 1;
+			if (ty >= c->height -1) {
+				ty = 1;
+			}
+		}
+	}
+	
+	// fallback vault location (or at least a non-crashy square)
+	p->px=vx;
+	p->py=vy;
+}
+
+/**
+ * Prepare the level the player is about to enter, either by generating
+ * or reloading
+ *
+ * \param c is the level we're going to end up with, in practice the global cave
+ * \param p is the current player struct, in practice the global player
+*/
+void prepare_next_level(struct chunk **c, struct player *p)
+{
+	/* Deal with any existing current level */
+	if (character_dungeon) {
+		assert (p->cave && (*c == cave));
+
+		if (OPT(p, birth_levels_persist)) {
+			/* Tidy up */
+			compact_monsters(0);
+			(*c)->squares[p->py][p->px].mon = 0;
+
+			/* Save level and known level */
+			cave_store(*c, false, true);
+			cave_store(p->cave, true, true);
+		} else {
+			/* Save the town */
+			if (!((*c)->depth) && !chunk_find_name("Town")) {
+				cave_store(*c, false, false);
+			}
+
+			/* Forget knowledge of old level */
+			if (p->cave && (*c == cave)) {
+				int x, y;
+
+				/* Deal with artifacts */
+				for (y = 0; y < (*c)->height; y++) {
+					for (x = 0; x < (*c)->width; x++) {
+						struct object *obj = square_object(*c, y, x);
+						while (obj) {
+							if (obj->artifact) {
+								bool found = obj->known && obj->known->artifact;
+								if (OPT(p, birth_lose_arts) || found) {
+									history_lose_artifact(p, obj->artifact);
+								} else {
+									obj->artifact->created = false;
+								}
+							}
+
+							obj = obj->next;
+						}
+					}
+				}
+
+				/* Free the known cave */
+				cave_free(p->cave);
+				p->cave = NULL;
+			}
+
+			/* Clear the old cave */
+			if (*c) {
+				cave_clear(*c, p);
+				*c = NULL;
+			}
+		}
+	}
+
+	/* Prepare the new level */
+	if (OPT(p, birth_levels_persist)) {
+		char *name = level_by_depth(p->depth)->name;
+		struct chunk *old_level = chunk_find_name(name);
+
+		/* If we found an old level, load the known level and assign */
+		if (old_level) {
+			int i;
+			char *known_name = format("%s known", name);
+			struct chunk *old_known = chunk_find_name(known_name);
+			assert(old_known);
+
+			/* Assign the new ones */
+			*c = old_level;
+			p->cave = old_known;
+
+			/* Associate known objects */
+			for (i = 0; i < p->cave->obj_max; i++) {
+				if ((*c)->objects[i] && p->cave->objects[i]) {
+					(*c)->objects[i]->known = p->cave->objects[i];
+				}
+			}
+
+			/* Allow monsters to recover */
+			restore_monsters();
+
+			/* Map boundary changes may not cooperate with level teleports */
+			sanitize_player_loc(*c,p);
+
+			/* Place the player */
+			player_place(*c, p, p->py, p->px);
+
+			/* Remove from the list */
+			chunk_list_remove(name);
+			chunk_list_remove(known_name);
+		} else {
+			/* Check dimensions */
+			struct level *lev;
+			int min_height = 0, min_width = 0;
+
+			/* Check level above */
+			lev = level_by_depth(p->depth - 1);
+			if (lev) {
+				struct chunk *check = chunk_find_name(lev->name);
+				if (check) {
+					get_min_level_size(check, &min_height, &min_width, true);
+				}
+			}
+
+			/* Check level below */
+			lev = level_by_depth(p->depth + 1);
+			if (lev) {
+				struct chunk *check = chunk_find_name(lev->name);
+				if (check) {
+					get_min_level_size(check, &min_height, &min_width, false);
+				}
+			}
+
+			/* Generate a new level */
+			*c = cave_generate(p, min_height, min_width);
+		}
+	} else {
+		/* Generate a new level */
+		*c = cave_generate(p, 0, 0);
+	}
+
+	/* Know the town */
+	if (!(p->depth)) {
+		cave_known(p);
+	}
 
 	/* The dungeon is ready */
-	character_dungeon = TRUE;
-
-	/* Free old and allocate new known level */
-	if (cave_k)
-		cave_free(cave_k);
-	cave_k = cave_new(cave->height, cave->width);
-	if (!cave->depth)
-		cave_known();
-
-	(*c)->created_at = turn;
+	character_dungeon = true;
 }
 
 /**
  * The generate module, which initialises template rooms and vaults
  * Should it clean up?
  */
-
 struct init_module generate_module = {
 	.name = "generate",
 	.init = run_template_parser,
