@@ -23,4 +23,6 @@ KF(SPELLS_OK,		"")
 KF(SHOOTS_SHOTS,	"")
 KF(SHOOTS_ARROWS,	"")
 KF(SHOOTS_BOLTS,	"")
+KF(RAND_BASE_RES,	"")
+KF(RAND_RES_POWER,	"")
 KF(MAX,				"")
