@@ -18,14 +18,15 @@
  */
 
 #include "angband.h"
-#include "cmds.h"
 #include "cave.h"
+#include "cmds.h"
 #include "init.h"
-#include "mon-util.h"
+#include "mon-predicate.h"
 #include "obj-ignore.h"
 #include "obj-util.h"
 #include "player-calcs.h"
 #include "player-path.h"
+#include "player-timed.h"
 #include "player-util.h"
 
 /**
@@ -55,7 +56,10 @@ static int dir_search[8] = {2,4,6,8,1,3,7,9};
 static bool is_valid_pf(int y, int x)
 {
 	/* Unvisited means allowed */
-	if (!square_ismark(cave, y, x)) return (TRUE);
+	if (!square_isknown(cave, y, x)) return (true);
+
+	/* No damaging terrain */
+	if (!square_isdamaging(cave, y, x)) return (true);
 
 	/* Require open space */
 	return (square_ispassable(cave, y, x));
@@ -84,7 +88,7 @@ static void fill_terrain_info(void)
 }
 
 #define MARK_DISTANCE(c,d) if ((c <= MAX_PF_LENGTH) && (c > d)) \
-							{ c = d; try_again = (TRUE); }
+							{ c = d; try_again = (true); }
 
 bool findpath(int y, int x)
 {
@@ -99,13 +103,12 @@ bool findpath(int y, int x)
 
 	if ((x >= ox) && (x < ex) && (y >= oy) && (y < ey)) {
 		if ((cave->squares[y][x].mon > 0) &&
-			mflag_has(square_monster(cave, y, x)->mflag, MFLAG_VISIBLE))
+			monster_is_visible(square_monster(cave, y, x))) {
 			terrain[y - oy][x - ox] = MAX_PF_LENGTH;
-
-		terrain[y - oy][x - ox] = MAX_PF_LENGTH;
+		}
 	} else {
 		bell("Target out of range.");
-		return (FALSE);
+		return (false);
 	}
 
 	/* 
@@ -113,7 +116,7 @@ bool findpath(int y, int x)
 	 * inefficient pathfinding algorithm
 	 */
 	do {
-		try_again = FALSE;
+		try_again = false;
 
 		for (j = oy + 1; j < ey - 1; j++) {
 			for (i = ox + 1; i < ex - 1; i++) {
@@ -134,7 +137,7 @@ bool findpath(int y, int x)
 		}
 
 		if (terrain[y - oy][x - ox] < MAX_PF_LENGTH)
-			try_again = FALSE;
+			try_again = false;
 
 	}
 	while (try_again) ;
@@ -142,7 +145,7 @@ bool findpath(int y, int x)
 	/* Failure */
 	if (terrain[y - oy][x - ox] == MAX_PF_LENGTH) {
 		bell("Target space unreachable.");
-		return (FALSE);
+		return (false);
 	}
 
 	/* Success */
@@ -169,7 +172,7 @@ bool findpath(int y, int x)
 
 	pf_result_index--;
 
-	return (TRUE);
+	return (true);
 }
 
 /**
@@ -371,12 +374,11 @@ int pathfind_direction_to(struct loc from, struct loc to)
 
 
 
-int run_cur_dir;		/* Direction we are running */
-int run_old_dir;		/* Direction we came from */
-bool run_unused;		/* Unused (padding field) */
-bool run_open_area;		/* Looking for an open area */
-bool run_break_right;	/* Looking for a break (right) */
-bool run_break_left;	/* Looking for a break (left) */
+static int run_cur_dir;		/* Direction we are running */
+static int run_old_dir;		/* Direction we came from */
+static bool run_open_area;		/* Looking for an open area */
+static bool run_break_right;	/* Looking for a break (right) */
+static bool run_break_left;	/* Looking for a break (left) */
 
 /**
  * Hack -- allow quick "cycling" through the legal directions
@@ -402,17 +404,17 @@ static int see_wall(int dir, int y, int x)
 	x += ddx[dir];
 
 	/* Illegal grids are not known walls XXX XXX XXX */
-	if (!square_in_bounds(cave, y, x)) return (FALSE);
+	if (!square_in_bounds(cave, y, x)) return (false);
 
 	/* Non-wall grids are not known walls */
 	if (!square_seemslikewall(cave, y, x))
-		return FALSE;
+		return false;
 
 	/* Unknown walls are not known walls */
-	if (!square_ismark(cave, y, x)) return (FALSE);
+	if (!square_isknown(cave, y, x)) return (false);
 
 	/* Default */
-	return (TRUE);
+	return (true);
 }
 
 
@@ -441,7 +443,7 @@ static void run_init(int dir)
 	bool shortleft, shortright;
 
 	/* Mark that we're starting a run */
-	player->upkeep->running_firststep = TRUE;
+	player->upkeep->running_firststep = true;
 
 	/* Save the direction */
 	run_cur_dir = dir;
@@ -450,15 +452,15 @@ static void run_init(int dir)
 	run_old_dir = dir;
 
 	/* Assume looking for open area */
-	run_open_area = TRUE;
+	run_open_area = true;
 
 	/* Assume not looking for breaks */
-	run_break_right = FALSE;
-	run_break_left = FALSE;
+	run_break_right = false;
+	run_break_left = false;
 
 	/* Assume no nearby walls */
-	deepleft = deepright = FALSE;
-	shortright = shortleft = FALSE;
+	deepleft = deepright = false;
+	shortright = shortleft = false;
 
 	/* Find the destination grid */
 	row = py + ddy[dir];
@@ -469,26 +471,26 @@ static void run_init(int dir)
 
 	/* Check for nearby or distant wall */
 	if (see_wall(cycle[i+1], py, px)) {
-		run_break_left = TRUE;
-		shortleft = TRUE;
+		run_break_left = true;
+		shortleft = true;
 	} else if (see_wall(cycle[i+1], row, col)) {
-		run_break_left = TRUE;
-		deepleft = TRUE;
+		run_break_left = true;
+		deepleft = true;
 	}
 
 	/* Check for nearby or distant wall */
 	if (see_wall(cycle[i-1], py, px)) {
-		run_break_right = TRUE;
-		shortright = TRUE;
+		run_break_right = true;
+		shortright = true;
 	} else if (see_wall(cycle[i-1], row, col)) {
-		run_break_right = TRUE;
-		deepright = TRUE;
+		run_break_right = true;
+		deepright = true;
 	}
 
 	/* Looking for a break */
 	if (run_break_left && run_break_right) {
 		/* Not looking for open area */
-		run_open_area = FALSE;
+		run_open_area = false;
 
 		/* Angled or blunt corridor entry */
 		if (dir & 0x01) {
@@ -509,7 +511,7 @@ static void run_init(int dir)
 /**
  * Update the current "run" path
  *
- * Return TRUE if the running should be stopped
+ * Return true if the running should be stopped
  */
 static bool run_test(void)
 {
@@ -551,28 +553,28 @@ static bool run_test(void)
 		/* Visible monsters abort running */
 		if (cave->squares[row][col].mon > 0) {
 			struct monster *mon = square_monster(cave, row, col);
-
-			/* Visible monster */
-			if (mflag_has(mon->mflag, MFLAG_VISIBLE)) return (TRUE);
+			if (monster_is_visible(mon)) {
+				return (true);
+			}
 		}
 
 		/* Visible objects abort running */
 		for (obj = square_object(cave, row, col); obj; obj = obj->next)
 			/* Visible object */
-			if (obj->marked && !ignore_item_ok(obj)) return (TRUE);
+			if (obj->known && !ignore_item_ok(obj)) return (true);
 
 		/* Assume unknown */
-		inv = TRUE;
+		inv = true;
 
 		/* Check memorized grids */
-		if (square_ismark(cave, row, col)) {
+		if (square_isknown(cave, row, col)) {
 			bool notice = square_isinteresting(cave, row, col);
 
 			/* Interesting feature */
-			if (notice) return (TRUE);
+			if (notice) return (true);
 
 			/* The grid is "visible" */
-			inv = FALSE;
+			inv = false;
 		}
 
 		/* Analyze unknown grids and floors */
@@ -585,10 +587,10 @@ static bool run_test(void)
 				option = new_dir;
 			} else if (option2) {
 				/* Three new directions. Stop running. */
-				return (TRUE);
+				return (true);
 			} else if (option != cycle[chome[prev_dir] + i - 1]) {
 				/* Two non-adjacent new directions.  Stop running. */
-				return (TRUE);
+				return (true);
 			} else if (new_dir & 0x01) {
 				/* Two new (adjacent) directions (case 1) */
 				option2 = new_dir;
@@ -601,10 +603,10 @@ static bool run_test(void)
 			if (run_open_area) {
 				if (i < 0) {
 					/* Break to the right */
-					run_break_right = TRUE;
+					run_break_right = true;
 				} else if (i > 0) {
 					/* Break to the left */
-					run_break_left = TRUE;
+					run_break_left = true;
 				}
 			}
 		}
@@ -625,13 +627,11 @@ static bool run_test(void)
 		if (row >= cave->height || col >= cave->width) continue;
 		if (row < 0 || col < 0) continue;
 
-		/* Visible monsters abort running */
+		/* Obvious monsters abort running */
 		if (cave->squares[row][col].mon > 0) {
 			struct monster *mon = square_monster(cave, row, col);
-			
-			/* Visible monster */
-			if (mflag_has(mon->mflag, MFLAG_VISIBLE) && !is_mimicking(mon))
-				return (TRUE);
+			if (monster_is_obvious(mon))
+				return (true);
 		}
 	}
 
@@ -646,16 +646,16 @@ static bool run_test(void)
 
 			/* Unknown grid or non-wall */
 			/* Was: square_ispassable(cave, row, col) */
-			if (!square_ismark(cave, row, col) ||
+			if (!square_isknown(cave, row, col) ||
 			    (square_ispassable(cave, row, col))) {
 				/* Looking to break right */
 				if (run_break_right) {
-					return (TRUE);
+					return (true);
 				}
 			} else { /* Obstacle */
 				/* Looking to break left */
 				if (run_break_left) {
-					return (TRUE);
+					return (true);
 				}
 			}
 		}
@@ -669,23 +669,23 @@ static bool run_test(void)
 
 			/* Unknown grid or non-wall */
 			/* Was: square_ispassable(cave, row, col) */
-			if (!square_ismark(cave, row, col) ||
+			if (!square_isknown(cave, row, col) ||
 			    (square_ispassable(cave, row, col))) {
 				/* Looking to break left */
 				if (run_break_left) {
-					return (TRUE);
+					return (true);
 				}
 			} else { /* Obstacle */
 				/* Looking to break right */
 				if (run_break_right) {
-					return (TRUE);
+					return (true);
 				}
 			}
 		}
 	} else { /* Not looking for open area */
 		/* No options */
 		if (!option) {
-			return (TRUE);
+			return (true);
 		} else if (!option2) { /* One option */
 			/* Primary option */
 			run_cur_dir = option;
@@ -704,11 +704,11 @@ static bool run_test(void)
 
 	/* About to hit a known wall, stop */
 	if (see_wall(run_cur_dir, py, px))
-		return (TRUE);
+		return (true);
 
 
 	/* Failure */
-	return (FALSE);
+	return (false);
 }
 
 
@@ -721,13 +721,17 @@ static bool run_test(void)
  */
 void run_step(int dir)
 {
+	/* Trapsafe player will treat the trap as if it isn't there */
+	bool disarm = player->timed[TMD_TRAPSAFE] ? false : true;
+
 	/* Start or continue run */
 	if (dir) {
 		/* Initialize */
 		run_init(dir);
 
-		/* Hack -- Set the run counter */
-		player->upkeep->running = 1000;
+		/* Hack -- Set the run counter if no count given */
+		if (player->upkeep->running == 0)
+			player->upkeep->running = 9999;
 
 		/* Calculate torch radius */
 		player->upkeep->update |= (PU_TORCH);
@@ -743,7 +747,7 @@ void run_step(int dir)
 		} else if (pf_result_index < 0) {
 			/* Pathfinding, and the path is finished */
 			disturb(player, 0);
-			player->upkeep->running_withpathfind = FALSE;
+			player->upkeep->running_withpathfind = false;
 			return;
 		} else {
 			int y = player->py + ddy[pf_result[pf_result_index] - '0'];
@@ -751,10 +755,10 @@ void run_step(int dir)
 
 			if (pf_result_index == 0) {
 				/* Known wall */
-				if (square_ismark(cave, y, x) &&
+				if (square_isknown(cave, y, x) &&
 					!square_ispassable(cave, y, x)) {
 					disturb(player, 0);
-					player->upkeep->running_withpathfind = FALSE;
+					player->upkeep->running_withpathfind = false;
 					return;
 				}
 			} else if (pf_result_index > 0) {
@@ -771,10 +775,10 @@ void run_step(int dir)
 				x = player->px + ddx[pf_result[pf_result_index] - '0'];
 
 				/* Known wall */
-				if (square_ismark(cave, y, x) &&
+				if (square_isknown(cave, y, x) &&
 					!square_ispassable(cave, y, x)) {
 					disturb(player, 0);
-					player->upkeep->running_withpathfind = FALSE;
+					player->upkeep->running_withpathfind = false;
 					return;
 				}
 
@@ -783,9 +787,9 @@ void run_step(int dir)
 					struct monster *mon = square_monster(cave, y, x);
 
 					/* Visible monster */
-					if (mflag_has(mon->mflag, MFLAG_VISIBLE)) {
+					if (monster_is_visible(mon)) {
 						disturb(player, 0);
-						player->upkeep->running_withpathfind = FALSE;
+						player->upkeep->running_withpathfind = false;
 						return;
 					}
 				}
@@ -793,9 +797,9 @@ void run_step(int dir)
 				/* Visible objects abort running */
 				for (obj = square_object(cave, y, x); obj; obj = obj->next)
 					/* Visible object */
-					if (obj->marked && !ignore_item_ok(obj)) {
+					if (obj->known && !ignore_item_ok(obj)) {
 					disturb(player, 0);
-					player->upkeep->running_withpathfind = FALSE;
+					player->upkeep->running_withpathfind = false;
 					return;
 				}
 
@@ -804,9 +808,9 @@ void run_step(int dir)
 				x = x + ddx[pf_result[pf_result_index - 1] - '0'];
 
 				/* Known wall, so run the direction we were going */
-				if (square_ismark(cave, y, x) &&
+				if (square_isknown(cave, y, x) &&
 					!square_ispassable(cave, y, x)) {
-					player->upkeep->running_withpathfind = FALSE;
+					player->upkeep->running_withpathfind = false;
 					run_init(pf_result[pf_result_index] - '0');
 				}
 			}
@@ -816,17 +820,18 @@ void run_step(int dir)
 		}
 	}
 
-	/* Decrease counter if it hasn't been cancelled */
-	if (player->upkeep->running)
-		player->upkeep->running--;
-	else if (!player->upkeep->running_withpathfind)
-		return;
-
 	/* Take time */
 	player->upkeep->energy_use = z_info->move_energy;
 
 	/* Move the player; running straight into a trap == trying to disarm */
-	move_player(run_cur_dir, dir ? TRUE : FALSE);
+	move_player(run_cur_dir, dir && disarm ? true : false);
+
+	/* Decrease counter if it hasn't been cancelled */
+	/* occurs after movement so that using p->u->running as flag works */
+	if (player->upkeep->running) {
+		player->upkeep->running--;
+	} else if (!player->upkeep->running_withpathfind)
+		return;
 
 	/* Prepare the next step */
 	if (player->upkeep->running) {
