@@ -24,6 +24,7 @@
 #include "obj-util.h"
 #include "player-calcs.h"
 #include "player-timed.h"
+#include "source.h"
 #include "trap.h"
 
 
@@ -33,7 +34,7 @@
  * ------------------------------------------------------------------------ */
 
 typedef struct project_feature_handler_context_s {
-	const int who;
+	const struct source origin;
 	const int r;
 	const int y;
 	const int x;
@@ -56,11 +57,11 @@ static void project_feature_handler_LIGHT_WEAK(project_feature_handler_context_t
 	if (square_isview(cave, y, x)) {
 		if (!player->timed[TMD_BLIND]) {
 			/* Observe */
-			context->obvious = TRUE;
+			context->obvious = true;
 		}
 
 		/* Fully update the visuals */
-		player->upkeep->update |= (PU_FORGET_VIEW | PU_UPDATE_VIEW | PU_MONSTERS);
+		player->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
 	}
 }
 
@@ -70,22 +71,18 @@ static void project_feature_handler_DARK_WEAK(project_feature_handler_context_t
 	const int x = context->x;
 	const int y = context->y;
 
-	if (player->depth != 0 || !is_daytime()) {
+	if ((player->depth != 0 || !is_daytime()) && !square_isbright(cave, y, x)) {
 		/* Turn off the light */
 		sqinfo_off(cave->squares[y][x].info, SQUARE_GLOW);
-
-		/* Hack -- Forget "boring" grids */
-		if (square_isfloor(cave, y, x))
-			sqinfo_off(cave->squares[y][x].info, SQUARE_MARK);
 	}
 
 	/* Grid is in line of sight */
 	if (square_isview(cave, y, x)) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 
 		/* Fully update the visuals */
-		player->upkeep->update |= (PU_FORGET_VIEW | PU_UPDATE_VIEW | PU_MONSTERS);
+		player->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
 	}
 }
 
@@ -96,76 +93,22 @@ static void project_feature_handler_KILL_WALL(project_feature_handler_context_t
 	const int y = context->y;
 
 	/* Non-walls (etc) */
-	if (square_ispassable(cave, y, x)) return;
+	if (square_ispassable(cave, y, x) && !square_seemslikewall(cave, y, x))
+		return;
 
 	/* Permanent walls */
 	if (square_isperm(cave, y, x)) return;
 
-	/* Granite */
-	if (square_iswall(cave, y, x) && !square_hasgoldvein(cave, y, x)) {
+	/* Different treatment for different walls */
+	if (square_isrubble(cave, y, x)) {
 		/* Message */
-		if (sqinfo_has(cave->squares[y][x].info, SQUARE_MARK)) {
-			msg("The wall turns into mud!");
-			context->obvious = TRUE;
-		}
-
-		/* Forget the wall */
-		sqinfo_off(cave->squares[y][x].info, SQUARE_MARK);
-
-		/* Destroy the wall */
-		square_destroy_wall(cave, y, x);
-	}
-
-	/* Quartz / Magma with treasure */
-	else if (square_iswall(cave, y, x) && square_hasgoldvein(cave, y, x))
-	{
-		/* Message */
-		if (sqinfo_has(cave->squares[y][x].info, SQUARE_MARK))
-		{
-			msg("The vein turns into mud!");
-			msg("You have found something!");
-			context->obvious = TRUE;
-		}
-
-		/* Forget the wall */
-		sqinfo_off(cave->squares[y][x].info, SQUARE_MARK);
-
-		/* Destroy the wall */
-		square_destroy_wall(cave, y, x);
-
-		/* Place some gold */
-		place_gold(cave, y, x, player->depth, ORIGIN_FLOOR);
-	}
-
-	/* Quartz / Magma */
-	else if (square_ismagma(cave, y, x) || square_isquartz(cave, y, x))
-	{
-		/* Message */
-		if (sqinfo_has(cave->squares[y][x].info, SQUARE_MARK))
-		{
-			msg("The vein turns into mud!");
-			context->obvious = TRUE;
-		}
-
-		/* Forget the wall */
-		sqinfo_off(cave->squares[y][x].info, SQUARE_MARK);
-
-		/* Destroy the wall */
-		square_destroy_wall(cave, y, x);
-	}
-
-	/* Rubble */
-	else if (square_isrubble(cave, y, x))
-	{
-		/* Message */
-		if (sqinfo_has(cave->squares[y][x].info, SQUARE_MARK))
-		{
+		if (square_isseen(cave, y, x)) {
 			msg("The rubble turns into mud!");
-			context->obvious = TRUE;
-		}
+			context->obvious = true;
 
-		/* Forget the wall */
-		sqinfo_off(cave->squares[y][x].info, SQUARE_MARK);
+			/* Forget the wall */
+			square_forget(cave, y, x);
+		}
 
 		/* Destroy the rubble */
 		square_destroy_rubble(cave, y, x);
@@ -174,118 +117,128 @@ static void project_feature_handler_KILL_WALL(project_feature_handler_context_t
 		if (randint0(100) < 10){
 			if (square_isseen(cave, y, x)) {
 				msg("There was something buried in the rubble!");
-				context->obvious = TRUE;
+				context->obvious = true;
 			}
-			place_object(cave, y, x, player->depth, FALSE, FALSE,
+			place_object(cave, y, x, player->depth, false, false,
 						 ORIGIN_RUBBLE, 0);
 		}
-	}
-
-	/* Destroy doors (and secret doors) */
-	else if (square_isdoor(cave, y, x))
-	{
+	} else if (square_isdoor(cave, y, x)) {
 		/* Hack -- special message */
-		if (sqinfo_has(cave->squares[y][x].info, SQUARE_MARK))
-		{
+		if (square_isseen(cave, y, x)) {
 			msg("The door turns into mud!");
-			context->obvious = TRUE;
-		}
+			context->obvious = true;
 
-		/* Forget the wall */
-		sqinfo_off(cave->squares[y][x].info, SQUARE_MARK);
+			/* Forget the wall */
+			square_forget(cave, y, x);
+		}
 
 		/* Destroy the feature */
 		square_destroy_door(cave, y, x);
+	} else if (square_hasgoldvein(cave, y, x)) {
+		/* Message */
+		if (square_isseen(cave, y, x)) {
+			msg("The vein turns into mud!");
+			msg("You have found something!");
+			context->obvious = true;
+
+			/* Forget the wall */
+			square_forget(cave, y, x);
+		}
+
+		/* Destroy the wall */
+		square_destroy_wall(cave, y, x);
+
+		/* Place some gold */
+		place_gold(cave, y, x, player->depth, ORIGIN_FLOOR);
+	} else if (square_ismagma(cave, y, x) || square_isquartz(cave, y, x)) {
+		/* Message */
+		if (square_isseen(cave, y, x)) {
+			msg("The vein turns into mud!");
+			context->obvious = true;
+
+			/* Forget the wall */
+			square_forget(cave, y, x);
+		}
+
+		/* Destroy the wall */
+		square_destroy_wall(cave, y, x);
+	} else if (square_iswall(cave, y, x)) {
+		/* Message */
+		if (square_isseen(cave, y, x)) {
+			msg("The wall turns into mud!");
+			context->obvious = true;
+
+			/* Forget the wall */
+			square_forget(cave, y, x);
+		}
+
+		/* Destroy the wall */
+		square_destroy_wall(cave, y, x);
 	}
 
 	/* Update the visuals */
 	player->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
-
-	/* Fully update the flow */
-	player->upkeep->update |= (PU_FORGET_FLOW | PU_UPDATE_FLOW);
 }
 
-/* Destroy Doors (and traps) */
+/* Destroy Doors */
 static void project_feature_handler_KILL_DOOR(project_feature_handler_context_t *context)
 {
 	const int x = context->x;
 	const int y = context->y;
 
-	/* Destroy all doors and traps */
-	if (square_isplayertrap(cave, y, x) || square_isdoor(cave, y, x))
-	{
+	/* Destroy all doors */
+	if (square_isdoor(cave, y, x)) {
 		/* Check line of sight */
-		if (square_isview(cave, y, x))
-		{
+		if (square_isview(cave, y, x)) {
 			/* Message */
 			msg("There is a bright flash of light!");
-			context->obvious = TRUE;
+			context->obvious = true;
 
 			/* Visibility change */
-			if (square_isdoor(cave, y, x))
-			{
-				/* Update the visuals */
-				player->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
-			}
+			player->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
+
+			/* Forget the door */
+			square_forget(cave, y, x);
 		}
 
-		/* Forget the door */
-		sqinfo_off(cave->squares[y][x].info, SQUARE_MARK);
-
 		/* Destroy the feature */
-		if (square_isdoor(cave, y, x))
-			square_destroy_door(cave, y, x);
-		else
-			square_destroy_trap(cave, y, x);
+		square_destroy_door(cave, y, x);
 	}
 }
 
-/* Destroy Traps (and Locks) */
+/* Disable traps, unlock doors */
 static void project_feature_handler_KILL_TRAP(project_feature_handler_context_t *context)
 {
 	const int x = context->x;
 	const int y = context->y;
 
 	/* Reveal secret doors */
-	if (square_issecretdoor(cave, y, x))
-	{
+	if (square_issecretdoor(cave, y, x)) {
 		place_closed_door(cave, y, x);
 
 		/* Check line of sight */
-		if (square_isview(cave, y, x))
-		{
-			context->obvious = TRUE;
-		}
+		if (square_isseen(cave, y, x))
+			context->obvious = true;
 	}
 
-	/* Destroy traps */
-	if (square_istrap(cave, y, x))
-	{
+	/* Disable traps, unlock doors */
+	if (square_istrap(cave, y, x)) {
 		/* Check line of sight */
-		if (square_isview(cave, y, x))
-		{
-			msg("There is a bright flash of light!");
-			context->obvious = TRUE;
+		if (square_isview(cave, y, x)) {
+			msg("The trap seizes up.");
+			context->obvious = true;
 		}
 
-		/* Forget the trap */
-		sqinfo_off(cave->squares[y][x].info, SQUARE_MARK);
-
-		/* Destroy the trap */
-		square_destroy_trap(cave, y, x);
-	}
-
-	/* Locked doors are unlocked */
-	else if (square_islockeddoor(cave, y, x))
-	{
+		/* Disable the trap */
+		square_disable_trap(cave, y, x);
+	} else if (square_islockeddoor(cave, y, x)) {
 		/* Unlock the door */
 		square_unlock_door(cave, y, x);
 
 		/* Check line of sound */
-		if (square_isview(cave, y, x))
-		{
+		if (square_isview(cave, y, x)) {
 			msg("Click!");
-			context->obvious = TRUE;
+			context->obvious = true;
 		}
 	}
 }
@@ -307,11 +260,11 @@ static void project_feature_handler_MAKE_DOOR(project_feature_handler_context_t
 		push_object(y,x);
 
 	/* Create closed door */
-	square_add_door(cave, y, x, TRUE);
+	square_add_door(cave, y, x, true);
 
 	/* Observe */
-	if (sqinfo_has(cave->squares[y][x].info, SQUARE_MARK))
-		context->obvious = TRUE;
+	if (square_isknown(cave, y, x))
+		context->obvious = true;
 
 	/* Update the visuals */
 	player->upkeep->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
@@ -327,9 +280,12 @@ static void project_feature_handler_MAKE_TRAP(project_feature_handler_context_t
 	if (!square_isempty(cave, y, x)) return;
 	if (square_iswarded(cave, y, x)) return;
 
-	/* Create a trap */
-	square_add_trap(cave, y, x);
-	context->obvious = TRUE;
+	/* Create a trap, try to notice it */
+	if (one_in_(4)) {
+		square_add_trap(cave, y, x);
+		(void) square_reveal_trap(cave, y, x, false, false);
+	}
+	context->obvious = true;
 }
 
 static void project_feature_handler_ACID(project_feature_handler_context_t *context)
@@ -338,7 +294,7 @@ static void project_feature_handler_ACID(project_feature_handler_context_t *cont
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -348,7 +304,7 @@ static void project_feature_handler_ELEC(project_feature_handler_context_t *cont
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -358,7 +314,18 @@ static void project_feature_handler_FIRE(project_feature_handler_context_t *cont
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
+	}
+
+	/* Can create lava if extremely powerful. */
+	if ((context->dam > randint1(1800) + 600) &&
+		square_isfloor(cave, context->y, context->x)) {
+		/* Forget the floor, make lava. */
+		square_unmark(cave, context->y, context->x);
+		square_set_feat(cave, context->y, context->x, FEAT_LAVA);
+
+		/* Objects that have survived should move */
+		push_object(context->y, context->x);
 	}
 }
 
@@ -368,7 +335,22 @@ static void project_feature_handler_COLD(project_feature_handler_context_t *cont
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
+	}
+
+	/* Sufficiently intense cold can solidify lava. */
+	if ((context->dam > randint1(900) + 300) &&
+		square_isfiery(cave, context->y, context->x)) {
+		bool occupied = square_isoccupied(cave, context->y, context->x);
+
+		square_unmark(cave, context->y, context->x);
+		if (one_in_(2)) {
+			square_set_feat(cave, context->y, context->x, FEAT_FLOOR);
+		} else if (one_in_(2) && !occupied) {
+			square_set_feat(cave, context->y, context->x, FEAT_RUBBLE);
+		} else {
+			square_set_feat(cave, context->y, context->x, FEAT_PASS_RUBBLE);
+		}
 	}
 }
 
@@ -378,7 +360,7 @@ static void project_feature_handler_POIS(project_feature_handler_context_t *cont
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -400,7 +382,7 @@ static void project_feature_handler_SOUND(project_feature_handler_context_t *con
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -410,7 +392,7 @@ static void project_feature_handler_SHARD(project_feature_handler_context_t *con
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -420,7 +402,7 @@ static void project_feature_handler_NEXUS(project_feature_handler_context_t *con
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -430,7 +412,7 @@ static void project_feature_handler_NETHER(project_feature_handler_context_t *co
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -440,7 +422,7 @@ static void project_feature_handler_CHAOS(project_feature_handler_context_t *con
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -450,7 +432,7 @@ static void project_feature_handler_DISEN(project_feature_handler_context_t *con
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -460,7 +442,7 @@ static void project_feature_handler_WATER(project_feature_handler_context_t *con
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -470,7 +452,22 @@ static void project_feature_handler_ICE(project_feature_handler_context_t *conte
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
+	}
+
+	/* Sufficiently intense cold can solidify lava. */
+	if ((context->dam > randint1(900) + 300) &&
+		square_isfiery(cave, context->y, context->x)) {
+		bool occupied = square_isoccupied(cave, context->y, context->x);
+
+		square_unmark(cave, context->y, context->x);
+		if (one_in_(2)) {
+			square_set_feat(cave, context->y, context->x, FEAT_FLOOR);
+		} else if (one_in_(2) && !occupied) {
+			square_set_feat(cave, context->y, context->x, FEAT_RUBBLE);
+		} else {
+			square_set_feat(cave, context->y, context->x, FEAT_PASS_RUBBLE);
+		}
 	}
 }
 
@@ -480,7 +477,7 @@ static void project_feature_handler_GRAVITY(project_feature_handler_context_t *c
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -490,7 +487,7 @@ static void project_feature_handler_INERTIA(project_feature_handler_context_t *c
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -500,7 +497,7 @@ static void project_feature_handler_FORCE(project_feature_handler_context_t *con
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -510,7 +507,7 @@ static void project_feature_handler_TIME(project_feature_handler_context_t *cont
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -520,7 +517,18 @@ static void project_feature_handler_PLASMA(project_feature_handler_context_t *co
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
+	}
+
+	/* Can create lava if extremely powerful. */
+	if ((context->dam > randint1(1800) + 600) &&
+		square_isfloor(cave, context->y, context->x)) {
+		/* Forget the floor, make lava. */
+		square_unmark(cave, context->y, context->x);
+		square_set_feat(cave, context->y, context->x, FEAT_LAVA);
+
+		/* Objects that have survived should move */
+		push_object(context->y, context->x);
 	}
 }
 
@@ -530,7 +538,7 @@ static void project_feature_handler_METEOR(project_feature_handler_context_t *co
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -540,7 +548,7 @@ static void project_feature_handler_MISSILE(project_feature_handler_context_t *c
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -550,7 +558,7 @@ static void project_feature_handler_MANA(project_feature_handler_context_t *cont
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -560,7 +568,7 @@ static void project_feature_handler_HOLY_ORB(project_feature_handler_context_t *
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
@@ -570,20 +578,93 @@ static void project_feature_handler_ARROW(project_feature_handler_context_t *con
 	if (square_isview(cave, context->y, context->x) &&
 		!player->timed[TMD_BLIND]) {
 		/* Observe */
-		context->obvious = TRUE;
+		context->obvious = true;
 	}
 }
 
+static void project_feature_handler_AWAY_UNDEAD(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_AWAY_EVIL(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_AWAY_ALL(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_TURN_UNDEAD(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_TURN_EVIL(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_TURN_ALL(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_DISP_UNDEAD(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_DISP_EVIL(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_DISP_ALL(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_CLONE(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_POLY(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_HEAL(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_SPEED(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_SLOW(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_CONF(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_SLEEP(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_HOLD(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_STUN(project_feature_handler_context_t *context)
+{
+}
+
+static void project_feature_handler_MON_DRAIN(project_feature_handler_context_t *context)
+{
+}
+
 static const project_feature_handler_f feature_handlers[] = {
-	#define ELEM(a, b, c, d, e, f, g, h, i, col) project_feature_handler_##a,
+	#define ELEM(a) project_feature_handler_##a,
 	#include "list-elements.h"
 	#undef ELEM
-	#define PROJ_ENV(a, col, desc) project_feature_handler_##a,
-	#include "list-project-environs.h"
-	#undef PROJ_ENV
-	#define PROJ_MON(a, obv, desc) NULL, 
-	#include "list-project-monsters.h"
-	#undef PROJ_MON
+	#define PROJ(a) project_feature_handler_##a,
+	#include "list-projections.h"
+	#undef PROJ
 	NULL
 };
 
@@ -593,12 +674,12 @@ static const project_feature_handler_f feature_handlers[] = {
  * Called for projections with the PROJECT_GRID flag set, which includes
  * beam, ball and breath effects.
  *
- * \param who is the monster list index of the caster
+ * \param origin is the origin of the effect
  * \param r is the distance from the centre of the effect
- * \param y
+ * \param y the coordinates of the grid being handled
  * \param x the coordinates of the grid being handled
  * \param dam is the "damage" from the effect at distance r from the centre
- * \param typ is the projection (GF_) type
+ * \param typ is the projection (PROJ_) type
  * \return whether the effects were obvious
  *
  * Note that this function determines if the player can see anything that
@@ -606,12 +687,12 @@ static const project_feature_handler_f feature_handlers[] = {
  *
  * Hack -- effects on grids which are memorized but not in view are also seen.
  */
-bool project_f(int who, int r, int y, int x, int dam, int typ)
+bool project_f(struct source origin, int r, int y, int x, int dam, int typ)
 {
-	bool obvious = FALSE;
+	bool obvious = false;
 
 	project_feature_handler_context_t context = {
-		who,
+		origin,
 		r,
 		y,
 		x,
