@@ -1,6 +1,6 @@
-/*
- * File: stats/structs.h
- * Purpose: data structures for lists not exported in headers elsewhere
+/**
+ * \file stats/structs.h
+ * \brief Data structures for lists not exported in headers elsewhere
  *
  * Copyright (c) 2011 Robert Au
  *
@@ -22,18 +22,17 @@
 #define STATS_STRUCTS_H
 
 #include "effects.h"
-#include "monster/monster.h"
-#include "monster/mon-spell.h"
-#include "object/slays.h"
+#include "monster.h"
+#include "mon-spell.h"
+#include "obj-slays.h"
 
-/*
+/**
  * Entries for spell/activation descriptions
  */
 typedef struct
 {
 	u16b index;          /* Effect index */
 	bool aim;            /* Whether the effect requires aiming */
-	u16b power;          /* Power rating for obj-power.c */
 	const char *desc;    /* Effect description */
 } info_entry;
 
