@@ -13,18 +13,21 @@ int setup_tests(void **state) {
 }
 
 int teardown_tests(void *state) {
+	struct object_kind *k = parser_priv(state);
+	string_free(k->name);
+	string_free(k->text);
+	mem_free(k);
 	parser_destroy(state);
 	return 0;
 }
 
 int test_name0(void *state) {
-	errr r = parser_parse(state, "name:3:Test Object Kind");
+	errr r = parser_parse(state, "name:Test Object Kind");
 	struct object_kind *k;
 
 	eq(r, 0);
 	k = parser_priv(state);
 	require(k);
-	eq(k->kidx, 3);
 	require(streq(k->name, "Test Object Kind"));
 	ok;
 }
