@@ -29,10 +29,10 @@
  * algorithm, used with permission. See below for copyright information
  * about the WELL implementation.
  *
- * To use of the "simple" RNG, activate it via "Rand_quick = TRUE" and
+ * To use of the "simple" RNG, activate it via "Rand_quick = true" and
  * "Rand_value = seed". After that it will be automatically used instead of
  * the "complex" RNG. When you are done, you can de-activate it via
- * "Rand_quick = FALSE". You can also choose a new seed.
+ * "Rand_quick = false". You can also choose a new seed.
  */
 
 /* begin WELL RNG
@@ -85,14 +85,14 @@ static u32b WELLRNG1024a (void){
 /**
  * Whether to use the simple RNG or not.
  */
-bool Rand_quick = TRUE;
+bool Rand_quick = true;
 
 /**
  * The current "seed" of the simple RNG.
  */
 u32b Rand_value;
 
-static bool rand_fixed = FALSE;
+static bool rand_fixed = false;
 static u32b rand_fixval = 0;
 
 /**
@@ -142,7 +142,7 @@ void Rand_init(void)
 #endif
 
 		/* Use the complex RNG */
-		Rand_quick = FALSE;
+		Rand_quick = false;
 
 		/* Seed the "complex" RNG */
 		Rand_state_init(seed);
@@ -316,6 +316,36 @@ s16b Rand_normal(int mean, int stand)
 }
 
 
+/**
+ * Choose an integer from a distribution where we know the mean and approximate
+ * upper and lower bounds.
+ *
+ * We divide the imagined distribution into two halves, above and below the
+ * mean, and then treat the bounds as if they are the given number of
+ * standard deviations from the mean in the appropriate direction.  Note that
+ * `stand_u` and `stand_l` are 10 times the number of standart deviations we
+ * are asking for.
+ * The function chooses an integer from a normal distribution, and then scales
+ * it to fit the target distribution.
+ */
+int Rand_sample(int mean, int upper, int lower, int stand_u, int stand_l)
+{
+	int pick = Rand_normal(0, 1000);
+
+	/* Scale to fit */
+	if (pick > 0) {
+		/* Positive pick, scale up */
+		pick *= (upper - mean);
+		pick /= (100 * stand_u);
+	} else if (pick < 0) {
+		/* Negative pick, scale down */
+		pick *= (mean - lower);
+		pick /= (100 * stand_l);
+	}
+
+	return mean + pick;
+}
+
 /**
  * Generates damage for "2d6" style dice rolls
  */
@@ -486,11 +516,11 @@ int randcalc(random_value v, int level, aspect rand_aspect)
 bool randcalc_valid(random_value v, int test)
 {
 	if (test < randcalc(v, 0, MINIMISE))
-		return FALSE;
+		return false;
 	else if (test > randcalc(v, 0, MAXIMISE))
-		return FALSE;
+		return false;
 	else
-		return TRUE;
+		return true;
 }
 
 /**
@@ -503,7 +533,7 @@ bool randcalc_varies(random_value v)
 
 void rand_fix(u32b val)
 {
-	rand_fixed = TRUE;
+	rand_fixed = true;
 	rand_fixval = val;
 }
 
@@ -518,6 +548,6 @@ u32b Rand_simple(u32b m)
 	static time_t seed;
 	time_t v;
 	v = time(NULL);
-	seed = LCRNG(seed) + ((v << 16) ^ v ^ getpid());
-	return (seed%m);
+	seed = LCRNG(seed % m) + ((v << 16) ^ v ^ getpid());
+	return (seed % m);
 }
