@@ -1,6 +1,6 @@
-/*
- * File: z-type.c
- * Purpose: Support various data types.
+/**
+ * \file z-type.c
+ * \brief Support various data types.
  *
  * Copyright (c) 2007 Angband Developers
  *
@@ -16,26 +16,9 @@
  *    are included in all such copies.  Other copyrights may also apply.
  */
 
-#include "h-basic.h"
-#include "z-form.h"
-#include "z-term.h"
-#include "ui.h"
 #include "z-type.h"
 #include "z-virt.h"
 
-#define TYPE_FUN(v2u, tv, T, v)	\
-type_union v2u(T v) { 		\
-	type_union r;			\
-	r.u.v = v; 				\
-	r.t = tv;				\
-	return r;				\
-}
-
-TYPE_FUN(i2u, T_INT, int, i)
-TYPE_FUN(c2u, T_CHAR, char, c)
-TYPE_FUN(f2u, T_FLOAT, float, f)
-TYPE_FUN(s2u, T_STRING, const char *, s)
-
 struct loc loc(int x, int y) {
 	struct loc p;
 	p.x = x;
@@ -43,7 +26,7 @@ struct loc loc(int x, int y) {
 	return p;
 }
 
-/*
+/**
  * Utility functions to work with point_sets
  */
 struct point_set *point_set_new(int initial_size)
@@ -61,7 +44,8 @@ void point_set_dispose(struct point_set *ps)
 	mem_free(ps);
 }
 
-/* Add the point to the given point set, making more space if there is
+/**
+ * Add the point to the given point set, making more space if there is
  * no more space left.
  */
 void add_to_point_set(struct point_set *ps, int y, int x)
